#!/usr/bin/perl -w

package GOGraph::GeneProduct::SequenceSimilarity::BlastSwissprot;

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;

use Bio::Tools::Run::RemoteBlast;
use Bio::Tools::Run::StandAloneBlast;
use Bio::Index::Swissprot;
use Bio::SeqIO;

use Config::App::AppProperties;


sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $self={};
  bless $self, $class;

  $self->{config} = Config::App::AppProperties::get_instance;
  $self->_ensure_index;

  return $self;
}


my $_indexfile;

sub _ensure_index{
  my $self = shift;

  if( defined $_indexfile ){
    $self->{inx} = $_indexfile;
    return;
  }

  my $properties = $self->{config}->load_config( "datasource.props" );
  my $_dat_file = $properties->getProperty( "swissprot.dat" )
    || "sprot.dat";

  $self->{inx_file} = $properties->getProperty( "swissprot.idx" )
    || "sprot.idx";

  my $inx = Bio::Index::Swissprot->new
    ('-filename'   => $self->{inx_file},
     '-write_flag' => 'WRITE' );

  $inx->make_index( $_dat_file );

  $self->{inx} = Bio::Index::Swissprot->new('-filename'
                                            => $self->{inx_file} );

  $self->{matrix} = $properties->getProperty( "smithwaterman.default.matrix" );

  $_indexfile = $self->{inx};


  return $self->{inx};
}

my $blast = 0;

sub blast_swissprot_acc{
  my $self = shift;
  my $accession = shift;

  my $seq = $self->{inx}->fetch( uc ( $accession ) );

  if( !defined $seq ){
    return undef;
  }

##  $blast++;
 ## print "Blasted $blast proteins \n";

  #return $self->blast_swissprot_ncbi( $seq );
  return $self->blast_swissprot_stand_alone( $seq );
}


sub blast_swissprot_stand_alone{
  my $self = shift;
  my $seq = shift;

  my @params = ('program'  => 'blastp',
                'database' => 'swissprot',
                '_READMETHOD' => 'BPlite');
  if( ! defined $self->{factory} ){
      $self->{factory} = Bio::Tools::Run::StandAloneBlast->new(@params);
  }

  my $factory = $self->{factory};

  ## set the expectation
  $factory->e( 100 );

  my @seqs;
  push @seqs, $seq;
  return $factory->blastall( @seqs );
}


sub blast_swissprot_ncbi{
  my $self = shift;
  my $seq = shift;
  #print "About to search with sequence...\n";
  #print $seq->seq . "\n";
  my $blastfactory = new Bio::Tools::Run::RemoteBlast ('-prog'      => "blastp",
                                                       '-data'      => "swissprot",
                                                       '-expect'    => "100",
                                                       'readmethod' => "BPlite",
                                                      );
  my @seqs;
  push @seqs, $seq;  
  $blastfactory->submit_blast( \@seqs );

  while( my @rids = $blastfactory->each_rid ){
    foreach my $rid (@rids){
      my $rc = $blastfactory->retrieve_blast($rid);
      if( !ref($rc) ) {
        if( $rc < 0 ) {
          # there was an error and this RID is not coming back
          $blastfactory->remove_rid($rid);
        }
        print ".";
        sleep 5;
      } else {
        #print "Return object is $rc\n";
        print "\n";
        $blastfactory->remove_rid($rid);
        return $rc;
      }
    }
  }
}


1;
