#!/usr/bin/perl -w

package GOGraph::GeneProduct::SmithWatermanLengthSimilarity;

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use Bio::Tools::pSW;


##
## Compare two sequences by aligning them with Smith Waterman and then
## taking the length of the aligned sequence
##

sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $self = {};

  $self->{config} = Config::App::AppProperties::get_instance;
  my $properties = $self->{config}->load_config( "datasource.props" );
  $self->{matrix} = $properties->getProperty( "smithwaterman.default.matrix" );
  $self->{factory} = new Bio::Tools::pSW( '-matrix' => $self->{matrix},
                                  '-gap' => 12,
                                  '-ext' => 2, );

  bless $self, $class;
  return $self;
}


sub similarity{
  my $self = shift;
  my $seq1 = shift;
  my $seq2 = shift;


  my $factory = $self->{factory};
  $aln = $factory->pairwise_alignment($seq1, $seq2);
  return $aln->length;
}


1;
