#!/usr/bin/perl -w

package GOGraph::GeneProduct::SequenceSimilarity::SwissprotSequenceSimilarity;

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use Bio::Index::Swissprot;
use Bio::SeqIO;
use Bio::Tools::pSW;

sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $sequence_similarity = shift;
  $self->{sequence_similarity} = $sequence_similarity;
}


sub _ensure_index{
  my $properties = $self->{config}->load_config( "datasource.props" );
  my $_dat_file = $properties->getProperty( "swissprot.dat" )
    || "sprot.dat";

  $self->{inx_file} = $properties->getProperty( "swissprot.idx" )
    || "sprot.idx";

  my $inx = Bio::Index::Swissprot->new
    ('-filename'   => $self->{inx_file},
     '-write_flag' => 'WRITE' );

  $inx->make_index( $_dat_file );
  $ensured_index = 1;

  $self->{inx} = Bio::Index::Swissprot->new('-filename'
                                            => $self->{inx_file} );

  $self->{matrix} = $properties->getProperty( "smithwaterman.default.matrix" );
  return $self->{inx};
}


sub sequence_similarity{
  my $self = shift;

  my $accession_one = shift;
  my $accession_two = shift;

  my $seq1 = $self->{inx}->fetch( uc ( $accession_one ) );
  my $seq2 = $self->{inx}->fetch( uc ( $accession_two ) );

  if( ( ! defined( $seq1 ) ) || ( ! defined( $seq2 ) ) ){
    print "Undefined sequence \n";
    return -1;
  }
  return $self->{sequence_similarity}->similarity( $seq1, $seq2 );
}

