#!/usr/bin/perl -w

=head1 NAME

  generateGoSwissproteinsAsFasta

=head1 DESCRIPTION

  This script generates a swissprot fasta file which reflects all of
  the swissprot sequences currently in the GO database. It does this
  by pulling out all the accession numbers/identifiers from the go
  database, and then the equivalent sequences from the sprot.dat file.

  I could do this by taking sequences from the go database directly,
  but this puts an extra step in between, which opens up possibilities
  of error.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use GOGraph::GeneProduct::AllSwissprotCollection;
use GOGraph::GeneProduct::SwissprotSequence;
use Bio::SeqIO;


print "Generating swissprot-human\n";

my $count = 0;

my $out = Bio::SeqIO->new('-file' => ">swissprot-human",
                         '-format' => 'Fasta');


my $sequence_producer = GOGraph::GeneProduct::SwissprotSequence->new();
my $sequence_ids = GOGraph::GeneProduct::AllSwissprotCollection->new()
  ->collection;


foreach $sequence_id ( @$sequence_ids ){
  my $sequence = $sequence_producer->sequence( $sequence_id );
  ## this is a hack. The files NCBI generate have a complex accession
  ## format. I pull the SP id out with a regexp, which will recognise
  ## this as well
  print "$count ";
  if( $count++ % 20 == 1 ){
    print "\n";
  }

  if( defined $sequence ){
    $sequence->display_id
      ( "sp|" . $sequence->display_id . "|" );
    $out->write_seq( $sequence );
  }
}

