#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::AcceptEvidence;

use strict;
## is this term accept or not?

=head2 accept

  Args - GO::Model::Term - is this term acceptable or not.
  Returns - true or false!

  Defines an interface for ensuring that a term is acceptable for
  Similarity testing.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $evidence_threshold = shift;

  my $self = {};
  $self->{evidence_threshold} = $evidence_threshold;

  bless $self, $class;
  return $self;
}

sub accept{
  my $self = shift;
  my $term = shift;

  ## this should select only those associations that we queried for in
  ## the first place.
  my $associations = $term->selected_association_list;

  ## Now have an association we want to pick out the one with the best
  ## evidence.
  my $evidence_score = $self->_best_evidence( $associations );

  return $evidence_score >= $self->{evidence_threshold};
}


sub _best_evidence{
  my $self = shift;
  my $association_list = shift;
  my $best_evidence = 0;
  my %probs = qw(E 0.1 P 0.1 IDS 0.1 IC 0.1
                 IEA 0.1 NAS 0.3 ND 0.3 NR  0.3 IEP 0.4 ISS 0.4
		 IMP 0.6 IGI 0.6 IPI 0.6 IDA 0.8
                 TAS 0.9 );

  foreach my $association (@$association_list){
    my $evidence_list = $association->evidence_list();
    foreach my $evidence (@$evidence_list){
      if( ! defined $probs{$evidence->code} ){
        print "undefined evidence code " . $evidence->code . " \n";
      }
      elsif( $probs{$evidence->code} > $best_evidence ){
        $best_evidence = $probs{$evidence->code};
      }
    }
  }
  return $best_evidence;
}

sub name{
  my $self = shift;

  return "evidence_above" . $self->{evidence_threshold};
}


1;
