#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::CachingSqlOccurrence;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::CachingSqlOccurrence

=head1 DESCRIPTION

  This class implements the Occurrence interface. It is a extension of
  the SqlOccurrence interface, and adds memory caching. Over time this
  increases the performance as all the data is held in memory.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};

use base qw{GOGraph::Similarity::InformationContent::Base::SqlOccurrence};


my $database_fetch = 0;
my $memory_fetch = 0;


sub _initialize{
  my $self = shift;

  $self->{occurrence_constraint} = {};
  $self->SUPER::_initialize();
}

sub _fetch_data_given_constraint{
  my $self = shift;
  my $constraint_name = shift;

  ## return the data if it exists, start an array other wise.
  my $data = $self->{occurrence_constraint}->{$constraint_name};

  if( defined $data ){
    return $data;
  }

  $data = {};
  $self->{occurrence_constraint}->{$constraint_name} = $data;
  return $data;
}


sub fetch_max_occurrences{
  my $self = shift;
  my $constraint_name = shift;

  my $data = $self->_fetch_data_given_constraint( $constraint_name );

  my $max_occurrences = $data->{max};
  if( defined $max_occurrences ){
    return $max_occurrences;
  }

  $data->{max} = $self->SUPER::fetch_max_occurrences( $constraint_name );
  return $data->{max};
}

sub fetch_occurrences{
  my $self = shift;
  my $constraint_name = shift;
  my $goaccn = shift;

  my $data = $self->_fetch_data_given_constraint( $constraint_name );


  my $occurrences = $data->{$goaccn};
  if( defined $occurrences ){
    $memory_fetch++;
    return $occurrences;
  }

  #if( ($database_fetch % 50) == 1 ){
  #  print "Similarity database cache " . $database_fetch . "\n";
  #}
  $database_fetch++;
  $data->{$goaccn} = $self->SUPER::fetch_occurrences( $constraint_name, $goaccn );
  return $data->{$goaccn};
}


1;
