#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::PrecachingSqlOccurrence;

=head1

  This does the same job as the CachingSqlOccurrence, but shoves
  everything into memory first. As far as I know, it DOES NOT WORK,
  properly, if at all. The idea is sound though. A single "select *
  from" statement is going to run a lot quicker than a lot of
  individual "select * from...where" statements. The flip side is that
  it slows start up time significantly, which is a pain for most
  applications, particularly when developing them, which is probably
  why this class never got finished.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};
use Carp;

use DBI;

use base qw{GOGraph::Similarity::InformationContent::Base::CachingSqlOccurrence};


my $_fetch_all_constraints=<<SQL;
select short_name from constraint_type
SQL

my $_fetch_all_occurrences=<<SQL;
select * from similarity where type_id = ?
SQL


sub _initialize{
  my $self = shift;

  $self->SUPER::_initialize();

  ## first we want to ensure all of the constraint data is filled.
  my $cons_sth = $self->{connection}->prepare( $_fetch_all_constraints );
  $cons_sth->execute;
  while( my $constraint = $cons_sth->fetchrow_arrayref ){
    my $constraint_name = $constraint->[ 0 ];
    print "Pre caching $constraint_name\n";
    ## actually where the precache data is stored
    my $occurrence_data = $self->_fetch_data_given_constraint( $constraint_name );
    ## other constraint data
    my $constraint_data = $self->_fetch_constraint_data( $constraint_name );

    my $occ_sth = $self->{connection}->prepare( $_fetch_all_occurrences );
    $occ_sth->execute( $constraint_data->{id} );

    while( my $res = $occ_sth->fetchrow_hashref ){
      $occurrence_data->{$res->{go_accn}} = $res->{occurences};
    }
  }
}

sub fetch_occurrences{
  my $self = shift;
  my $constraint_name = shift;
  my $goaccn = shift;


  my $data = $self->_fetch_data_given_constraint( $constraint_name );

  my $occurrences = $data->{$goaccn};

  if( defined $occurrences ){
    return $occurrences;
  }

  ## so its not been cached. So its zero...
  return 0;
}
