#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Lin98::LinSimilarity;

=head1 NAME

  GOGraph::Similarity::InformationContent::Lin98::LinSimilarity

=head1 DESCRIPTION

  Calculates semantic similarity based on the measure described by
  D. Lin in "An information-theoretic definition of similarity" from
  Proc. 15th International Conf. on Machine Learning, Morgan Kaufmann,
  San Francisco, CA page, 296-304 (1998)
  [http://citeseer.nj.nec.com/95071.html].

  For more details on usage see
  GOGraph::Similarity::InformationContent::Base::InformationContentSimilarity.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};
use Carp;


use base qw(GOGraph::Similarity::InformationContent::Base::InformationContentSimilarity);

sub constraint_name{
  my $self = shift;
  return "lin" . $self->SUPER::constraint_name();
}

sub _similarity{
  my $self = shift;
  my $rarest_occurrences = shift;
  my $first_term = shift;
  my $second_term = shift;

  my $probability_one = $self->{occurrence}->fetch_probability
    ( $self->{constraint_name}, $first_term->acc );
  my $probability_two = $self->{occurrence}->fetch_probability
    ( $self->{constraint_name}, $second_term->acc );

  my $probability_lso = $rarest_occurrences /
    $self->{occurrence}->fetch_max_occurrences( $self->{constraint_name} );

  my $divisor = log( $probability_one ) + log( $probability_two );

  if( $divisor == 0 ){
    return -1;
  }

  return
    (2 * log( $probability_lso ) ) /
      ( log( $probability_one ) + log( $probability_two ) );
}


1;
