#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Resnick99::ResnickSimilarity;

=head1 NAME

  GOGraph::Similarity::InformationContent::Resnick99::ResnickSimilarity

=head1 DESCRIPTION

  Calculates semantic similarity between two GO terms, based on the
  measure as described by Philip Resnik in:- "Semantic Similarity in a
  Taxonomy: An Information-Based Measure and its Application to
  Problems of Ambiguity in Natural Language", Journal of Artificial
  Intelligence Research 11:95-130 (1999)
  [http://citeseer.nj.nec.com/resnik99semantic.html]

  For usage description see
  GOGraph::Similarity::InformationContent::Base::InformationContent.

  I would like to extend my apologies to Philip Resnik here, for
  mis-spelling his surname (with an extra "c") in this class. I
  realise how irritating this can be, as my own first name (Phillip)
  gets mis-spelt all the time (with only one "l"). Perhaps this is why
  I normally use a diminutive version of it (Phil, spelt with one "l"
  just to confuse things). Anyway I'd like to fix it, but it'll create
  a lot of false diffs in the cvs, so I am not going to. 

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};
use Carp;

use GOGraph::Crawl::CrawlToRoot;
use GOGraph::Similarity::InformationContent::Base::Accept;
use GOGraph::Similarity::InformationContent::Base::OccurrenceFactory;

use base qw(GOGraph::Similarity::InformationContent::Base::InformationContentSimilarity);
#use diagnostics;




sub constraint_name{
  my $self = shift;
  return "resnick" . $self->SUPER::constraint_name();
}  

sub _similarity{
  my $self = shift;
  my $rarest_occurrences = shift;
  my $probability = $rarest_occurrences /
    $self->{occurrence}->fetch_max_occurrences( $self->{constraint_name} );

  ## This is a sanity checker. It will only happen if a term appears
  ## in the query that does not appear in corpus that was used to form
  ## the database. Under these circumstances we crash out.
  if( $probability == 0 ) {
    return -1;
  }
  return -log( $probability );
}



1;

