#!/usr/bin/perl -w

package GOGraph::Statistics::PrintSortedResults;


use Exporter;
use strict;
use vars qw(@ISA);
use base qw(GOGraph::Statistics::GOCompareOutputHandler Exporter);


use GOGraph::GeneProduct::SequenceSimilarity::SwissprotSequence;

my $instances = [];
my $sprot_dat_error_not_reported = 1;

sub _initialize{
  my $self = shift;

  if( defined $self->{comparitor_name} ){
    push @$instances, $self;

    $self->clear;
  }

  eval{
    $self->{swiss} = GOGraph::GeneProduct::SequenceSimilarity::SwissprotSequence->new();
  };

  if( !defined $self->{swiss} ){
    if( $sprot_dat_error_not_reported ){
      print "Sprot.dat not available. \n";
      print "The library will continue with reduced functionality\n";
      print "Original error was $@\n";
      $sprot_dat_error_not_reported = 0;
    }
  }
}

sub process_result{
  my $self = shift;
  my $semantic_similarity = shift;

  my $acc = shift;
  my $match_acc = shift;

  ## The results are stored in a hash, which is keyed on the semantic
  ## similarity, and has an arrayref value, which stores all of the
  ## accessions in with this value.
  my $result = $self->{result};
  unless( defined $result->{$semantic_similarity} ){
    $result->{$semantic_similarity} = [];
  }

  push @{$result->{$semantic_similarity}}, $match_acc;
}

sub clear{
  my $self = shift;
  $self->{result} = {};
}

sub print{
  my $self = shift;

  print "Similarity Score for " . $self->comparitor_name . "\n";
  my $cutoff = 0;
  foreach my $score ( reverse ( sort {$a <=> $b } ( keys %{$self->{result}} ) ) ){
    my $sequences = $self->{result}->{$score};
    foreach my $sequence ( @$sequences ){
      if ( $score <= 0 ){
        return;
      }
      if( $cutoff++ > 40 ){
        return;
      }

      my $desc = "";
      if( defined $self->{swiss} ){
        my $seq = $self->{swiss}->sequence( $sequence );
        $desc = substr( $seq->desc, 0, 60 );
      }
      print "$sequence, $desc has similarity $score \n";
    }
  }
}
sub clear_results{
  foreach my $inst ( @$instances ){
    $inst->clear;
  }
}


sub print_results{
  foreach my $inst ( @$instances ){
    $inst->print;
  }
}
