#!/usr/bin/perl -w

=head1 NAME

  CharacteriseRelationships

=head1 DESCRIPTIONS

  Counts the number of nodes with different children...

=cut

use strict;

use GOGraph::GOAppHandleFactory;
use GOGraph::Statistics::Scripts::CharacteriseRelationships;
use GO::AppHandle;#
use Config::App::AppProperties;

my $apph = GOGraph::GOAppHandleFactory::go_apph();



my $root_terms = $apph->get_ontology_root_terms();
push @$root_terms, $apph->get_root_term;


print "Characterise children\n";
foreach my $start_term ( @$root_terms ){
  count_children( $start_term );
}




sub count_children{
  my $term = shift;
  print "For the root term " . $term->name . "\n";

  my $results= {};
  $results->{"both"}   = 0;
  $results->{"isa"}    = 0;
  $results->{"partof"} = 0;
  $results->{"leaf"} = 0;

  my $seen_terms = {};
  count_children_1( $results, $term, $seen_terms );

  use Dumpvalue;
  Dumpvalue->new()->dumpValue( $results );
}


sub count_children_1{
  my $results = shift;
  my $term = shift;
  my $seen_terms = shift;

  #print "on term " . $term->name . "\n";
  ## check that we have not seen this term before...

  if( !defined( $seen_terms->{$term->public_acc} ) ){

    $seen_terms->{$term->public_acc} = $term;
    ## fetch all relationships
    my $relationships = $apph->get_relationships({parent=>$term});
    ## check them all
    my $is_isa = 0;
    my $is_partof = 0;
    my $relationship_count = 0;
    
    foreach my $relationship (@$relationships){
      my $accn = $relationship->acc2;
      my $child_term = $apph->get_term({acc=>$accn});
      ##print "\ton relationship to " .  $child_term->name .  "\n";
      ##print "\t\thas type " . $relationship->type . "\n";

      $relationship_count++;
      
      my $type = $relationship->type;
      if( $type eq "is_a" ){
        $is_isa = 1;
      }
      if( $type eq "part_of" ){
        $is_partof = 1;
      }
    }


    ##print "\tend isa $is_isa and  partof $is_partof\n";
    if( $is_isa && $is_partof ){
      ##print "\tSo its both\n";
      $results->{"both"}++;
    }
    elsif( $is_isa ){
      ##print "\tSo its isa\n";
      $results->{"isa"}++;
    }
    elsif( $is_partof ){
      ##print "\tSo its partof\n";
      $results->{"partof"}++;
    }
    elsif( $relationship_count gt 0 ){
      print "****Problem***\n";
    }
    else{
      ##print "\tSo its a leaf\n";
      $results->{"leaf"}++;
    }
    ## recurse over children..
    foreach my $relationship (@$relationships){
      my $child_acc = $relationship->acc2;
      my $child_term = $apph->get_term({acc=>$child_acc});

      count_children_1( $results, $child_term, $seen_terms );
    }
  }
}
