#!/usr/bin/perl -w

=head1 NAME

  CharacteriseNots

=head1 DESCRIPTION

  This class characterises the number of negated associations between
  GO terms and Gene Products.

  I was interested in this, because my semantic similarity measures do
  not take account of negation at all. In facts it's deeply unclear
  how it should be taken account of. Fortunately (in swissprot
  anyway), there are not used, so it doesn't make any difference
  anyway.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut
use GOGraph::GOAppHandleFactory;
use GOGraph::GeneProduct::AllSwissprotCollection;
use GOGraph::GeneProduct::AllCollection;

$| = 1;
my $apph = GOGraph::GOAppHandleFactory::go_apph();

my $evidences = {};

my $ident = GOGraph::GeneProduct::AllSwissprotCollection->new()
  ->collection();


print "Analysing swissprot for evidence tags\n";
$evidences->{swissprot} = analyse( $ident );

$ident = GOGraph::GeneProduct::AllCollection->new()->collection();

print "Analysing everything for evidence tags\n";
$evidences->{all} = analyse( $ident );

use Dumpvalue;
Dumpvalue->new->dumpValue( $evidences );

sub analyse{

  my $ident = shift;

  # shorten everything
#  my @blah = @$ident;
#  $#blah = 1000;
#  $ident = \@blah;
  my $number = 0;
  foreach my $accession ( @$ident ){
    my $terms = $apph->get_terms({'product'=>$accession});
    foreach my $term ( @$terms ){
      my $associations = $term->selected_association_list;
      foreach my $association (@$associations){
        if( $association->is_not ){
          print "\n" . $term->acc . " " . $association->gene_product->acc;
          $number++;
        }
      }
    }
    print ".";
  }

  print "\n";
  return $number;
}
