#!/usr/bin/perl -w

=head1 NAME

  CharacteriseRelationships

=head1 DESCRIPTIONS

  Counts the number of different relationships, for
  the different aspects of GO.

=cut

use strict;

use GOGraph::GOAppHandleFactory;
use GOGraph::Statistics::Scripts::CharacteriseRelationships;
use GO::AppHandle;



my $apph = GOGraph::GOAppHandleFactory::go_apph();

my $root_terms = $apph->get_ontology_root_terms();

push @$root_terms, $apph->get_root_term;

my $answer = "";

foreach my $start_term ( @$root_terms ){

  my $count = count_all_children( $start_term, 0, {});
  $answer = $answer . "Name: " . $start_term->name . " Count:" . $count . "\n";
  print "Name: " . $start_term->name . " Count:" . $count . "\n";
  sleep( 10 );
}

print $answer;


sub count_all_children{
  my $term = shift;
  my $i = shift;
  my $seen = shift;

  $i ++;

  $seen->{ $term->public_acc } = 1;

  print "Counting children of " . $term->name . "\n";

  my $child_terms = $term->get_child_terms;

  if( scalar( @$child_terms ) == 0 ){
    ##print "\tTerminating at " . $term->name . "\n";
    return $i;
  }

  foreach my $term ( @$child_terms ){
    if( defined $seen->{ $term->public_acc } ){
      print "\t\tAlready seen " . $term->name . "\n";
    }
    else{
      ##print "\t\tRecursing from " . $term->name . "\n";
      $i = count_all_children( $term, $i, $seen );
    }
  }
  return $i;
}
