#!/usr/bin/perl -w

package GOGraph::Statistics::CharacteriseRelationshipsMod

=head1 NAME

  GOGraph::Statistics::CharacteriseRelationships

=head1 SYNOPSIS

  my $no = GOGraph::Statistics::CharacteriseRelationships
  ::characterise_relationships( $apph->get_root_term );

  Dumpvalue->new()->dumpValue( $results );


=head1 DESCRIPTION

  Characterises the number of the different types of relationship
  beneath a given node.

=cut

use strict;



### Given a term work out what terms below it are of what type
sub characterise_relationships{
  my $term = shift;

  ## hash to store the relationships that we have seen already
  my $seen_relationships = {};
  my $result_hash = {};
  return _characterise_relationships( $term, $seen_relationships, $result_hash );
}

sub _characterise_relationships{
  my $term = shift;

  ## only count each relationship once. You can get to a relationship
  ## by multiple paths so this is a concern.
  my $seen_relationships = shift;
  ## Store the results in here!
  my $result_hash = shift;

  my $apph = $term->apph;

  ## fetch all child relationships
  my $relationships = $apph->get_relationships({parent=>$term});
  foreach my $relationship (@$relationships){
    my $child_acc = $relationship->acc2;
    my $child_term = $apph->get_term({acc=>$child_acc});

    my $relationship_id = $term->acc . $child_acc;

    ## check whether we have not seen this relationship before
    if( !defined( $seen_relationships->{$relationship_id} ) ){
      ## recurse down the tree
      _characterise_relationships( $child_term, $seen_relationships, $result_hash );
      ## now we have seen this relationship
      $seen_relationships->{$relationship_id} = 1;
      ## and add characterisation of this relationship
      $result_hash->{$relationship->type}++;
    }
  }
  return $result_hash;
}



1;
