#!/usr/bin/perl -w

package GOGraph::Statistics::Scripts::CountDepths;

=head1 NAME

  GOGraph::Statistics::Scripts::CountDepths

=head1 DESCRIPTION

  Prints longest path from a given root. For usage see CountDepth.pl script.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;


sub count_depths{
  my $term = shift;

  my $path = [];

  _descend_tree( $term, $path, 0, 10000 );
}


sub _descend_tree{
  my $term = shift;
  my $path = shift;
  my $max = shift;
  my $min = shift;

  push @$path, $term;

  my $apph = $term->apph;

  ## fetch all child relationships
  my $relationships = $apph->get_relationships({parent=>$term});

  if( scalar @$relationships == 0 ){
    if( scalar @$path > $max ){
      $max = scalar @$path;
      print "Max path is $max\n";
      dump_path( $path );
    }

    if( scalar @$path < $min ){
      $min = scalar @$path;
      print "Min path is $min\n";
      dump_path( $path );
    }

    pop @$path;
    return [$max, $min];
  }

  foreach my $relationship (@$relationships){
    my $child_acc = $relationship->acc2;
    my $child_term = $apph->get_term({acc=>$child_acc});

    ($max, $min) = @{_descend_tree( $child_term, $path, $max, $min )};
  }
  pop @$path;
  return [$max, $min];
}


sub dump_path{
  my $path = shift;

  foreach my $term( @$path ){
    print $term->name . "\n";
  }
  print "\n";
}

1;

