#!/usr/bin/perl -w


=head1 NAME

  EvidenceProbabilities

=head1 DESCRIPTION

  This class seemed like a good idea at the time, but probably wasn't.

  The idea was to ask whether the different evidence codes had different
  information content averages. I'm not sure this is anyway meaningful.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

  use strict;

use GOGraph::Similarity::Factory;
use GOGraph::Similarity::InformationContent::Base::OccurrenceFactory;
use GOGraph::Similarity::InformationContent::Base::PrecachingSqlOccurrence;
use GOGraph::GeneProduct::Collections::AllSwissprotCollection;

$| = 1;

my $apph = GOGraph::GOAppHandleFactory::go_apph();
my $constraints = GOGraph::Similarity::Factory::constraints_array_reference();

my $occurrence =
  GOGraph::Similarity::InformationContent::Base::CachingSqlOccurrence->new();


my $ident = GOGraph::GeneProduct::Collections::AllSwissprotCollection->new()->collection();

#my @blah = @$ident;
#$#blah = 100;
#$ident = \@blah;

analyse( $ident );

sub analyse{
  my $ident = shift;

  my $total = {};
  my $number = {};

  foreach my $constraint ( @$constraints ) {
    my $cons_name = $constraint->_get_description_from_constraints();
    $total->{$cons_name}  = {};
    $number->{$cons_name} = {};
  }


  foreach my $accession ( @$ident ) {
    my $terms = $apph->get_terms({'product'=>$accession});
    foreach my $term ( @$terms ) {
      my $associations = $term->selected_association_list;
      foreach my $association (@$associations) {
        my $evidence_list = $association->evidence_list();
        foreach my $evidence (@$evidence_list) {
          foreach my $constraint ( @$constraints ) {
            my $cons_name = $constraint->_get_description_from_constraints();
            my $prob = $occurrence->fetch_probability( $cons_name, $term->acc );
            if ( $prob >= 0 ) {
              $total->{$cons_name}->{$evidence->code} += $prob;
              $number->{$cons_name}->{$evidence->code}++;
            }
          }
        }
      }
    }
    print STDERR ".";
  }

  dumpvalues( $total, $number );
}

sub dumpvalues{
  my $total = shift;
  my $number = shift;

  foreach my $constraint ( @$constraints ) {
    my $cons_name = $constraint->_get_description_from_constraints();
    print "$cons_name\n";
    foreach my $ev ( keys( %{$total->{$cons_name}} )){
      my $tot = $total->{$cons_name}->{$ev};
      my $num = $number->{$cons_name}->{$ev};
      if ( $num > 1 ) {
        my $ave = $tot/$num;
        print "\t$ev $ave\n";
      }
    }
  }
}
